#include "RaveBase/Converters/interface/RaveStreamers.h"
#include <iomanip>
#include <iostream>

using namespace std;

namespace
{
  bool isDiagonal ( const rave::Covariance3D & cov )
  {
    static const float min = 1.e-11;
    if ( fabs ( cov.dxy() ) > min ) return false;
    if ( fabs ( cov.dxz() ) > min ) return false;
    if ( fabs ( cov.dyz() ) > min ) return false;
    return true;
  }
}

ostream& operator<< ( ostream& os, const rave::Point3D & p )
{
  os << "(" << setprecision(4) << fixed << p.x() << ", " << p.y() << ", " << p.z() << ")";
  return os;
}

ostream& operator<< ( ostream& os, const rave::Covariance3D & p )
{
  #define sep " "<<std::setw(12)
  os << std::setprecision(4) << std::scientific << std::right
     << "(" << sep << p.dxx()  << sep << p.dxy()  << sep << p.dxz()  << std::endl
     << "(" << sep << p.dxy()  << sep << p.dyy()  << sep << p.dyz()  << std::endl
     << "(" << sep << p.dxz()  << sep << p.dyz()  << sep << p.dzz()  << ")" << std::endl;
  return os;
}

ostream& operator<< ( ostream& os, const rave::Covariance6D & p )
{
  /*
  os << "(" << setprecision(7) << p.dxx() << endl 
     << " " << p.dxy() << " " << p.dyy() << endl
     << " " << p.dxz() << " " << p.dyz() << " " << p.dzz() << ")" << endl;
  os << "(" << p.dpxpx() << endl 
     << " " << p.dpxpy() << " " << p.dpypy() << endl
     << " " << p.dpxpz() << " " << p.dpypz() << " " << p.dpzpz() << ")" << endl;
  os << "(" << p.dxpx() << " " << p.dxpy() << " " << p.dxpz() << endl 
     << " " << p.dypx() << " " << p.dypy() << " " << p.dypz() << endl
     << " " << p.dzpx() << " " << p.dzpy() << " " << p.dzpz() << ")" << endl;
     */
  os << "(" << setprecision(7) << scientific << p.dxx() << "     " << p.dxy() << "       " << p.dxz() 
     << "       " << p.dxpx() << "      " << p.dxpy() << "      " << p.dxpz() << endl

     << "(" << p.dxy() << "        " << p.dyy() << "       " << p.dyz() << "       " << p.dypx() 
     << "      " << p.dypy() << "      " << p.dypz() << endl

     << "(" << p.dxz() << "        " << p.dyz() << "       " << p.dzz() << "       " << p.dzpx() 
     << "      " << p.dzpy() << "      " << p.dzpz() << endl

     << "(" << p.dxpx() << "       " << p.dypx() << "      " << p.dzpx() << "      " << p.dpxpx() 
     << "     " << p.dpxpy() << "     " << p.dpxpz() << endl

     << "(" << p.dxpy() << "       " << p.dypy() << "      " << p.dzpy() << "      " << p.dpxpy() 
     << "     " << p.dpypy() << "     " << p.dpypz() << endl

     << "(" << p.dxpz() << "       " << p.dypz() << "      " << p.dzpz() << "      " << p.dpxpz() 
     << "     " << p.dpypz() << "     " << p.dpzpz() << ")" << endl;
  return os;
}

ostream& operator<< ( ostream& os, const rave::Ellipsoid3D & p )
{
  rave::Point3D pt = p.point();
  const rave::Covariance3D & cov = p.covariance();
  /*
  if ( isDiagonal ( cov ) )
  {
    os << "(" << setprecision(4) << fixed << pt.x() << "+/-" << sqrt ( cov.dxx() )
    << ", " << pt.y() << "+/-" << sqrt ( cov.dyy() )
    << ", " << pt.z() << "+/-" << sqrt ( cov.dzz() )
    << ")";
  }
  else*/
  {
    os << setprecision(4) << fixed << pt << endl << "Cov=" << endl << scientific << cov;
  }
  return os;
}


/*
ostream& operator<< (ostream& os, const rave::Vector3D & p )
{
  os << "(" << p.x() << ", " << p.y() << ", " << p.z() << ")";
  return os;
}*/

ostream& operator<< ( ostream& os, const rave::Vector6D & p )
{
  os << "(" << setprecision(6) << fixed << p.x() << ", " << p.y() << ", " << p.z() << " , "
  << p.px() << ", " << p.py() << ", " << p.pz() << ")";
  return os;
}

ostream& operator<< ( ostream& os, const rave::Track & t )
{
  os << t.state();
  return os;
}

ostream& operator<< ( ostream& os, const rave::Vertex & v )
{
  os << "id: " << v.id() << ", position: " << v.position();
  return os;
}
